<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 08, 2021
 */
require_once __DIR__ . '/AcronisDbInfo.php';
/**
 * @package InterWorx
 * @subpackage CLI
 */
class PreBackup extends CLI
{
    const FUNCTION_CACHE = 'cacheServerInfo';
    const PARAM_UNMAPPED_FOLDER = 'unmapped_dir';
    const CACHE_BACKUP_DIR = '/home/.acronis_tmp';
    const CACHE_BACKUP_FILE = self::CACHE_BACKUP_DIR . '/sw_cache.json';

    const COLUMN_UNIQNAME = 'uniqname';

    private $_Cache;

    /**
     * Register functions, install & uninstall.
     */
    public function registerOptArgs()
    {
        $this->_func(self::FUNCTION_CACHE, 'cache backup info');
        $this->_param(self::PARAM_UNMAPPED_FOLDER . '=s', 'name of unmapped database folder');
    }

    /**
     * save start time, unixusers & db info
     */
    protected function _cacheServerInfo()
    {
        $timestamp = time();

        $prescriptData = array(
            Model_Cache_Abstract::MAPPED_DATABASES => $this->_getMappedDatabases(),
            Model_Cache_Abstract::UNMAPPED_DATABASES => $this->_getUnmappedDatabases(),
            Model_Cache_Abstract::SW_USERS => $this->_getAllUnixusers(),
            Model_Cache_Abstract::TIMESTAMP => $timestamp,
            Model_Cache_Abstract::DATE => Model_Acronis::getDateString($timestamp),
            Model_Cache_Abstract::DUMPS_DIR_UNMAPPED => $this->_getOption(self::PARAM_UNMAPPED_FOLDER)
        );

        // add file so that cache can be recreated from backup
        IWorxFileSys::mkdir(self::CACHE_BACKUP_DIR);
        IWorxFileSys::smartBarf(self::CACHE_BACKUP_FILE, json_encode($prescriptData));

        $Cache = $this->_getCache();
        $Cache->add($timestamp, $prescriptData);
    }

    /**
     * @return Model_Cache_Prescript
     */
    private function _getCache()
    {
        if (!isset($this->_Cache)) {
            $this->_Cache = new Model_Cache_Prescript();
        }

        return $this->_Cache;
    }

    /**
     * @return array
     */
    private function _getMappedDatabases()
    {
        $jsonPath = AcronisDbInfo::JSON_FILE_LOCATION_SW;

        if (Model_AcronisFileSys::isFile($jsonPath)) {
            return $this->_getDbsFromJson();
        } else {
            $users =$this->_getUnixUsersWithDumps();
            return $this->_getDbsFromDumps($users);
        }
    }

    /**
     * Get all unix users that has dumps.
     *
     * Dumps folder is created for all unix users
     * at the time of the dumps creation.
     *
     * @return array
     */
    private function _getUnixUsersWithDumps()
    {
        $filtered = array_filter($this->_getAllUnixusers(), function ($user) {
            return $this->_userExists($user);
        });

        return array_values($filtered);
    }

    /**
     * Get all unix users.
     */
    private function _getAllUnixusers()
    {
        $NW = new NodeWorx(NodeWorx::MASTER_ID);
        $Search = new IWorx_Search_SiteworxAccounts($NW);
        $Search->addSelectableFields(self::COLUMN_UNIQNAME);

        $format = IWorx_Search_SiteworxAccounts::FETCH_ASSOC;
        $users = $Search->search($format);

        return array_map(function ($user) {
            return $user[self::COLUMN_UNIQNAME];
        }, $users);
    }

    /**
     * @param string $unixuser
     * @return boolean
     */
    private function _userExists($unixuser)
    {
        $userDumpsPath = Model_AcronisFileSys::getUserDumpsPath($unixuser);
        return Model_AcronisFileSys::isDir($userDumpsPath);
    }



    /**
     * Get dbs from json file. Only include dbs that
     */
    private function _getDbsFromJson()
    {
        return IWorx_Json::readFromFile(AcronisDbInfo::JSON_FILE_LOCATION_SW, true);
    }

    /**
     * @param array $users
     * @return array
     */
    private function _getDbsFromDumps(array $users)
    {
        $dbs = array();

        foreach ($users as $user) {
            $dbs = array_merge($dbs, $this->_getUserDbsFromDumps($user));
        }

        return $dbs;
    }

    /**
     * @param string $unixuser
     * @return array
     */
    private function _getUserDbsFromDumps($unixuser)
    {
        $dumpsPath = Model_Recovery_Files_Databases::DUMPS_TEMP_PATH . "/$unixuser-dumps";
        $dbs = array();

        if (Model_AcronisFileSys::isDir($dumpsPath)) {
            $dbs = $this->_getUserContent($dumpsPath, $unixuser);
        }

        return $dbs;
    }

    /**
     * Get folder content
     *
     * Dumps folder should contain folders named
     * after dbs.
     *
     * @param string $path
     * @param string $unixuser
     * @return array
     */
    private function _getUserContent($path, $unixuser)
    {
        $content = Model_AcronisFileSys::getDirContent($path);
        $result = array();

        foreach ($content as $db) {
            $result[] = array(
                Model_Cache_Abstract::NAME => $db,
                Model_Cache_Abstract::UNIXUSER => $unixuser
            );
        }

        return $result;
    }

    /**
     * @return array
     */
    private function _getUnmappedDatabases()
    {
        $NW = new NodeWorx(NodeWorx::MASTER_ID);
        $Driver  = $this->_getMySQLDriver();
        $DbMapper = SiteworxDbMapper::getInstance();

        $dbs = array();
        foreach ($Driver->getServers() as $Server) {
            $params = array(
                'database_server' => $Server,
                'nodeworx'        => $NW
            );

            $ServerDriver = new MySQL($params);
            $unmapped = $DbMapper->getUnmappedDatabases($ServerDriver);

            foreach ($unmapped as $dbName) {
                $dbs[] = array(Model_Cache_Abstract::NAME => $dbName);
            }
        }

        return $dbs;
    }

    /**
     * @return MySQL
     */
    private function _getMySQLDriver()
    {
        $NW = new NodeWorx(NodeWorx::MASTER_ID);
        return $NW->getMySQLDriver();
    }
}
