<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Aug 17, 2021
 */

/**
 * @package InterWorx
 * @subpackage
 */
abstract class Model_Cache_Restore extends Model_Cache_Abstract
{
    /**
     * add to cache or replace if already exists.
     *
     * @param string $id
     * @param array  $restore
     */
    public function add($id, array $restore)
    {
        $restores = $this->getAll();
        $restores[$id] = $restore;
        $this->_set($restores);
    }

    /**
     * @param string $user
     * @return array
     */
    public function getRestoresByUnixUser($user)
    {
        $restores = $this->getAll();

        $result = array();
        foreach ($restores as $restore) {
            $this->_validateUnixUserCached($restore);

            if ($restore[self::UNIXUSER] === $user) {
                $result[] = $restore;
            }
        }

        return $result;
    }

    /**
     * @param array $restore
     */
    private function _validateUnixUserCached(array $restore)
    {
        if (is_null($restore[self::UNIXUSER])) {
            throw new Exception('Acronis Cache: unixuser not cached!');
        }
    }

    /**
     * Started = db dumps are being recovered
     *
     * @param string $id
     */
    public function setStarted($id)
    {
        $this->_validateId($id, __FUNCTION__);
        $this->_updateState($id, self::STATE_STARTED);
    }

    /**
     * @param string $id
     * @param string $state
     */
    private function _updateState($id, $state)
    {
        $restores = $this->getAll();
        $restores[$id][self::STATE] = $state;
        $this->_set($restores);
    }

    /**
     * In progress = db dumps are restored and we have started the recovery
     *
     * @param string $id
     */
    public function setInProgress($id)
    {
        $this->_validateId($id, __FUNCTION__);
        $this->_updateState($id, self::STATE_IN_PROGRESS);
    }



    /**
     * Cancelled = an error occurred & we could not complete db recovery.
     *
     * @param string $id
     */
    public function setCancelled($id)
    {
        $this->_validateId($id, __FUNCTION__);
        $this->_updateState($id, self::STATE_CANCELLED);
    }

    /**
     * Complete = no error occurred, db is recovered.
     *
     * @param string $id
     */
    public function setComplete($id)
    {
        $this->_validateId($id, __FUNCTION__);
        $this->_updateState($id, self::STATE_COMPLETED);
    }

    /**
     * @param string $id
     */
    public function setFailed($id)
    {
        $this->_validateId($id, __FUNCTION__);
        $this->_updateState($id, self::STATE_FAILED);
    }

    /**
     * Update planId for specified restore item.
     *
     * @param string $id
     * @param string $planId
     */
    public function setPlanId($id, $planId)
    {
        $this->_validateId($id, __FUNCTION__);

        $restores = $this->getAll();
        $restores[$id][self::PLAN_ID] = $planId;
        $this->_set($restores);
    }

    /**
     * @param string $id
     * @param string $error
     */
    public function setErrorMsg($id, $error)
    {
        $this->_validateId($id, __FUNCTION__);

        $restores = $this->getAll();
        $restores[$id][self::ERROR] = $error;
        $this->_set($restores);
    }

    /**
     * check if any restore is in progress or started for user.
     *
     * @param string $unixUser
     */
    public function userHasActiveRestores($unixUser)
    {
        $restores = $this->getRestoresByUnixUser($unixUser);
        foreach ($restores as $restore) {
            if ($this->_restoreIsActive($restore)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Checks if any restore is set to 'started' or 'in progress'
     *
     * @return boolean
     */
    public function restoresInProgress()
    {
        foreach ($this->getAll() as $restore) {
            if ($this->_restoreIsActive($restore)) {
                return true;
            }
        }

        return false;
    }

    /**
     * @param array $restore
     */
    private function _restoreIsActive(array $restore)
    {
        $restoreInProgress = $restore[self::STATE] === self::STATE_IN_PROGRESS;
        $restoreStarted = $restore[self::STATE] === self::STATE_STARTED;

        return $restoreInProgress || $restoreStarted;
    }
}
