<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jan 12, 2021
 */

/**
 * @package InterWorx
 * @subpackage Plugin
 */
class Model_RecoveryPlan_Plans extends Model_Acronis
{
    const PLAN_ID = 'planId';
    const RECOVERY_ID = 'recoveryId';
    const STATE = 'state';

    /**
     * Get all recovery plans.
     *
     * @return array
     */
    public function getAll()
    {
        $tasks = $this->getTasksApi()->getRecoveries();
        return $this->_formatRecoveries($tasks);
    }

    /**
     * @param string $id
     * @return Model_RecoveryPlan_Plan
     */
    public function getRecoveryPlanById($id)
    {
        $Api = $this->getTasksApi();

        $count = 0;
        $Task = $Api->getRecoveryByPolicyId($id);

        // may take a few seconds before api is updated
        while ($Task->getId() === null && $count < 5) {
            $count++;
            sleep(5);

            $Task = $Api->getRecoveryByPolicyId($id);
        }

        if ($Task->getId() === null) {
            throw new Exception('unable to retrieve plan from api');
        }

        return new Model_RecoveryPlan_Plan($Task);
    }

    /**
     * Check if plan exists
     *
     * @param string $id
     */
    public function planExists($id): bool
    {
        try {
            // will throw exception if planid not found in api
            $this->getRecoveryPlanById($id);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * @param string $unixuser
     */
    public function getByUnixUser($unixuser)
    {
        $Cache = new Model_Cache_Restore_Files();
        return $Cache->getByUser($unixuser);
    }

    /**
     * @param Model_ApiResponse_Task[] $tasks
     * @return array
     */
    private function _formatRecoveries(array $tasks)
    {
        $plans = array();

        foreach ($tasks as $Task) {
            $plans[] = array(
                self::RECOVERY_ID => $Task->getUuid(),
                self::PLAN_ID => $Task->getPolicyId(),
                self::STATE => $Task->getState()
            );
        }

        return $plans;
    }

    /**
     * resume recovery.
     *
     * @param string $uuid
     */
    public function resumeRecovery($uuid)
    {
        $this->_validatePlan($uuid);
        $this->getActivitiesApi()->ignoreErrorsAndResumeRecovery($uuid);
    }

    /**
     * @param string $uuid
     */
    private function _validatePlan($uuid)
    {
        $plan = $this->_getExistingPlan($uuid);
        $state = $plan['state'];

        if ($state !== 'paused') {
            throw new Exception_Acronis_ResumeRecovery($state);
        }
    }

    /**
     * Gets list of plans from acronis api and returns plan with matching uuid.
     * Throws exception if no plan with matching uuid exists.
     *
     * @param string $uuid
     * @return array
     */
    private function _getExistingPlan($uuid)
    {
        $plans = $this->getAll();

        foreach ($plans as $plan) {
            if ($plan['recoveryId'] === $uuid) {
                return $plan;
            }
        }

        throw new Exception_Acronis_InvalidInput("Plan with recovery id does not exist: " . $uuid);
    }
}
