<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2021 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package    InterWorx
 * @author     Malin Lorch <mlorch@liquidweb.com>
 * @date       Jul 23, 2021
 */

require_once(__DIR__ . './../../../Model/Cache/Backup.php');
/**
 * @package InterWorx
 * @subpackage
 */
class Cache_BackupTest extends IWorx_Framework_TestCase
{

    private $_backupId = 'somebackupid';
    private $_SUT;

    /**
     * Create SUT & add to cache
     */
    public function setUp()
    {
        $this->_SUT = new Model_Cache_Backup();
    }

    /**
     * Delete test item from cache
     */
    public function tearDown()
    {
        $this->_getSut()->delete($this->_backupId);
    }

    /**
     * test add. Should add timestamp and date.
     */
    public function testAdd()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $actual = $SUT->get($this->_backupId);

        $this->assertArrayHasKey($SUT::BACKUP_ID, $actual);
        $this->assertEquals($this->_backupId, $actual[$SUT::BACKUP_ID]);
    }

    /**
     * Get cache data.
     *
     * @return array
     */
    private function _getCacheData()
    {
        return array(
            Model_Cache_Abstract::API_DETAILS => $this->_getApiDetails(),
            Model_Cache_Abstract::BACKUP_ID => $this->_backupId
        );
    }

    /**
     * @return array
     */
    private function _getApiDetails()
    {
        return array(
            Model_Backups_Backup::DATE => 'date',
            Model_Backups_Backup::BACKUP_ID => $this->_backupId,
            Model_Backups_Backup::LEGACY_ID => 'legacyId',
            Model_Backups_Backup::VAULT_ID => 'vaultId',
            Model_Backups_Backup::ARCHIVE_ID => 'archiveId',
            Model_Backups_Backup::MACHINE_ID => 'machineId',
            Model_Backups_Backup::CREATED => 'created',
            Model_Backups_Backup::TENANT_AGENT_ID => 'tenantId'
        );
    }

    /**
     * Test addSwUsers
     */
    public function testAddSwUsers()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getSwUsers($this->_backupId));

        $expected = array('user1', 'user2');
        $SUT->addSwUsers($this->_backupId, $expected);

        $this->assertEquals($expected, $SUT->getSwUsers($this->_backupId));
    }

    /**
     * Test addMappedDatabases
     *
     * should throw exception if array does not contain arrays.
     */
    public function testAddMappedDatabasesShouldThrowExceptionIInvalidContent()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getMappedDatabases($this->_backupId));

        $expected = array('mapped1', 'mapped2');

        $this->expectException(Exception::class);
        $SUT->addMappedDatabases($this->_backupId, $expected);
    }

    /**
     * Test addMappedDatabases
     *
     * should throw exception if key 'name' is missing.
     */
    public function testAddMappedDatabasesShouldThrowExceptionIfNameMissing()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getMappedDatabases($this->_backupId));

        $expected = array(array(
            'unixuser' => 'mapped1'
        ));

        $this->expectException(Exception::class);
        $SUT->addMappedDatabases($this->_backupId, $expected);
    }

    /**
     * Test addMappedDatabases
     *
     * should throw exception if key 'unixuser' is missing.
     */
    public function testAddMappedDatabasesShouldThrowExceptionIfUnixuserMissing()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getMappedDatabases($this->_backupId));

        $expected = array(array(
            'name' => 'mapped1'
        ));

        $this->expectException(Exception::class);
        $SUT->addMappedDatabases($this->_backupId, $expected);
    }

    /**
     * Test addMappedDatabases
     *
     * Should add dbs
     */
    public function testAddMappedDatabasesShouldSucceed()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getMappedDatabases($this->_backupId));

        $expected = array(array(
            'name' => 'mapped1',
            'unixuser' => 'unix'
        ));

        $SUT->addMappedDatabases($this->_backupId, $expected);
        $this->assertEquals($expected, $SUT->getMappedDatabases($this->_backupId));
    }


    /**
     * Test addUnmappedDatabases
     *
     * Should throw exception if array does not contain arrays
     */
    public function testAddUnmappedDatabasesShouldThrowExceptionIfInvalidStructure()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getUnmappedDatabases($this->_backupId));

        $expected = array('unmapped1');

        $this->expectException(Exception::class);
        $SUT->addUnmappedDatabases($this->_backupId, $expected);
    }

    /**
     * Test addUnmappedDatabases
     *
     * Should throw exception if key 'name' is missing
     */
    public function testAddUnmappedDatabasesShouldThrowExceptionIfNameMissing()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getUnmappedDatabases($this->_backupId));

        $expected = array(array(
            'n' => 'unmapped1'
        ));

        $this->expectException(Exception::class);
        $SUT->addUnmappedDatabases($this->_backupId, $expected);
    }

    /**
     * Test addUnmappedDatabases
     *
     * Should succeed when structure is as expected
     */
    public function testAddUnmappedDatabasesShouldSucceed()
    {
        $SUT = $this->_getSut();
        $SUT->add($this->_backupId, $this->_getCacheData());

        $this->assertNull($SUT->getUnmappedDatabases($this->_backupId));

        $expected = array(array(
            'name' => 'unmapped1'
        ));

        $SUT->addUnmappedDatabases($this->_backupId, $expected);
        $this->assertEquals($expected, $SUT->getUnmappedDatabases($this->_backupId));
    }

    /**
     * @return Model_Cache_Backup
     */
    private function _getSut()
    {
        return $this->_SUT;
    }
}
